<?php

/**
 * @package DeMomentSomTres Restaurant
 */
/*
 * Plugin Name: DMS3 Restaurant
 * Plugin URI: http://www.demomentsomtres.com/en/wordpress-plugins/demomentsomtres-restaurant/
 * Description: DMS3 Restaurants creates a custom type to represent restaurant lists and menus and show them using shortcodes and menu entries.
 * Version: 202105030000
 * Author: DeMomentSomTres
 * Author URI: http://www.DeMomentSomTres.com
 * License: GPLv2 or later
 * Text Domain: demomentsomtres-restaurant
 * Domain Path: /languages
 */

require_once (dirname(__FILE__) . '/lib/class-tgm-plugin-activation.php');
require_once dirname(__FILE__) . '/lib/dompdf/lib/html5lib/Parser.php';
require_once dirname(__FILE__) . '/lib/dompdf/lib/php-font-lib/src/FontLib/Autoloader.php';
require_once dirname(__FILE__) . '/lib/dompdf/lib/php-svg-lib/src/autoload.php';
require_once dirname(__FILE__) . '/lib/dompdf/src/Autoloader.php';
Dompdf\Autoloader::register();

// reference the Dompdf namespace
use Dompdf\Dompdf;
use Dompdf\Options;

$demomentsomtres_restaurant = new DeMomentSomTresRestaurant;

class DeMomentSomTresRestaurant {

    const OPTIONS = 'dms3-restaurant';
    const OPTIONS2 = 'dms3-restaurant_options';
    const POSTTYPE = 'dish-list';
    const EXPIRYDATE = "expiry-date";
    const DATAAMAGARFINS = "amagar-fins";
    const MENULIFE = "menu-life";
    const TEMPLATES = "number-of-templates";
    const TEMPLATETITLE = "title";
    const TEMPLATECONTENT = "content";
    const TEMPLATEPRINTSTYLE = "pdf-print-style";
    const PRINTLOGO = "pdf-logo";
    const OLDEXPIRYDATE = "demomentsomtres-restaurant-expiry-date";
    const TAXONOMY = "dish-list-type";
    const ACTION = "action";
    const PRINTMENU = "demomentsomtres-printmenu";
    const PDFOUTPUT = "pdfoutput";
    const OPTION_WC_BLOCKS = "woocommerce-blocks";
    const OPTION_WC_PRODUCTS_BLOCK = "woocommerce-products-per-block";
    const FIELD_WC_PRODUCTS = "woocommerce-products";
    const SHORTCODE_WC = "demomentsomtres-restaurant-woocommerce-block";

    private $pluginURL;
    private $pluginPath;
    private $langDir;

    function __construct() {
        $this->pluginURL = plugin_dir_url(__FILE__);
        $this->pluginPath = plugin_dir_path(__FILE__);
        $this->langDir = dirname(plugin_basename(__FILE__)) . '/languages';

        add_action('plugins_loaded', array($this, 'plugin_loaded'));
        add_action('tgmpa_register', array($this, 'required_plugins'));
        add_action('init', array($this, 'posttypes'), 1);
        add_action('after_setup_theme', array($this, "reduxOptions"));
        add_action("rwmb_meta_boxes", array($this, "do_meta_boxes"));
        add_action("admin_enqueue_scripts", array($this, "admin_enqueue_scripts"));
        add_action('admin_head', array($this, 'insert_buttons'));
        // add_action("wp_footer",array($this,"wp_footer"));
        add_action("wp_ajax_dms3_restaurant_update", array($this, "webserviceUpgrade"));
        add_action("wp_ajax_nopriv_dms3_restaurant_update", array($this, "webserviceUpgrade"));
        add_action("wp_enqueue_scripts", array($this, "enqueue_scripts"));
        add_shortcode('eco', array($this, 'shortcode_eco'));
        add_shortcode('veg', array($this, 'shortcode_veg'));
        add_shortcode('cel', array($this, 'shortcode_cel'));
        add_shortcode('P', array($this, 'shortcode_p'));
        add_shortcode('demomentsomtres-restaurant-dish-list', array($this, 'shortcode_dish_list'));
        add_shortcode('demomentsomtres-restaurant-type', array($this, 'shortcode_type'));
        add_shortcode('demomentsomtres-restaurant-post', array($this, 'shortcode_post'));
        add_action('the_content', array($this, 'content_expired_filter'));
        add_shortcode(self::SHORTCODE_WC, array($this, 'shortcode_woocommerce'));

        add_action("template_redirect", array($this, "template_redirect_printmenu"), 30);
    }

    function plugin_loaded() {
        load_plugin_textdomain("demomentsomtres-restaurant", false, $this->langDir);
    }

    function required_plugins() {
        $plugins = array(
            array(
                "name" => "Redux Framework",
                "slug" => "redux-framework",
                "required" => true
            ),
            array(
                "name" => 'Meta Box',
                "slug" => 'meta-box',
                "required" => true,
            ),
        );
        tgmpa($plugins);
    }

    function reduxOptions() {
        if (!class_exists('Redux')) {
            return;
        }

        $redux = get_option(self::OPTIONS);

        Redux::setArgs(self::OPTIONS, Array(
            'opt_name' => self::OPTIONS,
            'menu_title' => __("DMS3 Restaurant", "demomentsomtres-restaurant"),
            'display_name' => __("DeMomentSomTres Restaurant Settings", "demomentsomtres-restaurant"),
            'menu_type' => 'submenu',
            'page_parent' => "edit.php?post_type=" . self::POSTTYPE,
            'admin_bar' => false,
            'dev_mode' => false,
            'footer_text' => __("<br/><strong>More information</strong>", "demomentsomtres-restaurant") .
            "<br/><a href='//www.demomentsomtres.com/en/wordpress-plugins/demomentsomtres-restaurant/' target='_blank'>Plugin page</a>",
        ));

        $mainfields = array(
            array(
                "type" => "slider",
                "title" => __("Menu Life", "demomentsomtres-restaurant"),
                "desc" => __("Days that a menu will be alive before getting expired, by default", "demomentsomtres-restaurant"),
                "id" => self::MENULIFE,
                "default" => 1,
                'min' => 1,
                'max' => 365,
            ),
            array(
                "type" => "slider",
                "title" => __("Number of templates", "demomentsomtres-restaurant"),
                "desc" => __("Number of templates that you want to mantain.", "demomentsomtres-restaurant") .
                "<br/><strong>" . __("WARNING:", "demomentsomtres-restaurant") . "</strong>" .
                __("If you decrease this number, templates can be lost.", "demomentsomtres-restaurant"),
                "id" => self::TEMPLATES,
                "default" => 1,
                'max' => 500,
            ),
        );

        if (class_exists('WooCommerce')):
            $mainfields[] = array(
                "type" => "slider",
                "title" => __("WooCommerce Blocks", "demomentsomtres-restaurant"),
                "desc" => __("Number of WooCommerce product blocks in the menu.", "demomentsomtres-restaurant") .
                "<br/><strong>" . __("WARNING:", "demomentsomtres-restaurant") . "</strong>" .
                __("If you decrease this number, products in dishlist can be lost.", "demomentsomtres-restaurant"),
                "id" => self::OPTION_WC_BLOCKS,
                "default" => 0,
                'max' => 500,
            );
            $mainfields[] = array(
                "type" => "slider",
                "title" => __("Max products in WooCommerce Blocks", "demomentsomtres-restaurant"),
                "desc" => __("Number of WooCommerce products in each block in the menu.", "demomentsomtres-restaurant") .
                "<br/><strong>" . __("WARNING:", "demomentsomtres-restaurant") . "</strong>" .
                __("If you decrease this number, products in dishlist can be lost.", "demomentsomtres-restaurant"),
                "id" => self::OPTION_WC_PRODUCTS_BLOCK,
                "default" => 0,
                'max' => 500,
            );
        endif;

        Redux::setSection(self::OPTIONS, array(
            "title" => __("Main settings", "demomentsomtres-restaurant"),
            'fields' => $mainfields,
        ));

        $ntemp = $redux[self::TEMPLATES];
        if ($ntemp == 0):
            $ntemp = 1;
        endif;

        $optionfields = array(
            array(
                "type" => "info",
                "title" => __("Instructions", "demomentsomtres-restaurant"),
            ),
            array(
                "type" => "info",
                "id" => "note1",
                "title" => __("Add a new template", "demomentsomtres-restaurant"),
                "desc" => __("Go to main settings and increase the number of templates", "demomentsomtres-restaurant"),
            ),
            array(
                "type" => "info",
                "id" => "note2",
                "title" => __("Delete last template", "demomentsomtres-restaurant"),
                "desc" => __("Go to main settings and decrease the number of templates", "demomentsomtres-restaurant"),
            ),
            array(
                "type" => "info",
                "id" => 'note3',
                "title" => __("Delete a template not at bottom", "demomentsomtres-restaurant"),
                "desc" => __("Not implemented. You can copy content and title of previous templates and delete the last one.", "demomentsomtres-restaurant"),
            ),
        );

        for ($i = 1; $i <= $ntemp; $i++):
            $optionfields[] = array(
                "type" => "info",
                "title" => __("Template #", "demomentsomtres-restaurant") . $i,
            );
            $optionfields[] = array(
                "type" => "text",
                "title" => __("Title", "demomentsomtres-restaurant"),
                "id" => self::TEMPLATETITLE . "-" . $i,
            );
            $optionfields[] = array(
                "type" => "editor",
                "title" => __("Content", "demomentsomtres-restaurant"), "id" => self::TEMPLATECONTENT . "-" . $i,
                "lang" => "html",
            );
        endfor;

        Redux::setSection(self::OPTIONS, array(
            "title" => __("Templates", "demomentsomtres-restaurant"),
            'icon' => 'el el-list-alt',
            'fields' => $optionfields,
        ));

        Redux::setSection(self::OPTIONS, array(
            "title" => __("PDF Style", "demomentsomtres-restaurant"),
            'icon' => 'el el-paper-clip',
            'fields' => array(
                array(
                    "type" => "ace_editor",
                    "title" => __("CSS for printable menu", "demomentsomtres-restaurant"),
                    "id" => self::TEMPLATEPRINTSTYLE,
                    "lang" => "css",
                ),
                array(
                    "type" => "media",
                    "title" => __("Logo", "demomentsomtres-restaurant"),
                    "id" => self::PRINTLOGO,
                ),
                array(
                    "type" => "switch",
                    "title" => __("PDF Output", "demomentsomtres-restaurant"),
                    "id" => self::PDFOUTPUT,
                    'on' => 'Enabled',
                    'off' => 'Disabled'
                ),
            )
        ));

        if ($redux['button-old-config'] != 1) :
            Redux::setSection(self::OPTIONS, array(
                "title" => __("Migració", "demomentsomtres-restaurant"),
                'icon' => 'el el-adjust-alt',
                "fields" => array(
                    array(
                        'id' => 'button-old-config',
                        'type' => 'button_set',
                        'desc' => "<strong>" . __("WARNING:", "demomentsomtres-restaurant") . "</strong>" .
                        __(" Si cliques el botó i tornes a carregar la pàgina, perdràs tota la configuració que tenies fins ara."),
                        'options' => array(
                            '1' => 'Configuració anterior',
                        ),
                    ),
            )));
        endif;

        if (class_exists('WooCommerce')):
            $nwcb = $redux[self::OPTION_WC_BLOCKS];
            $nwcpb = $redux[self::OPTION_WC_PRODUCTS_BLOCK];
            for ($i = 0; $i < $nwcb; $i++):
                $metaBoxWC[$i] = array(
                    "title" => __("WooCommerce Product Block", "demomentsomtres-restaurant") . " " . ($i + 1),
                    "post_type" => array(self::POSTTYPE),
                    "priority" => "high",
                );
                for ($j = 0; $j < $nwcpb; $j++):
                    $metaBoxWC[$i] = array(
                        "type" => "select-posts",
                        "id" => self::FIELD_WC_PRODUCTS . "-$i-$j",
                        "title" => __("Product", "demomentsomtres-restaurant") . " " . ($j + 1),
                        "default" => 0,
                        "post_type" => "product",
                    );
                endfor;
            endfor;
        endif;
    }

    function enqueue_scripts() {
        wp_register_script("dms3Restaurant", $this->pluginURL . "js/demomentsomtres-restaurant.js", "jquery", true);
        wp_enqueue_script("dms3Restaurant");
    }

    function webserviceUpgrade() {
        global $wpdb;
        $success = true;

        $old = get_option(self::OPTIONS2);
        $new = maybe_unserialize(get_option(self::OPTIONS));

        $replace = maybe_unserialize(preg_replace('/0/', 'a', $old, 1));

        $new['title-1'] = $replace['title-1'];
        $new['title-2'] = $replace['title-2'];
        $new['title-3'] = $replace['title-3'];

        $new['content-1'] = $replace['content-1'];
        $new['content-2'] = $replace['content-2'];
        $new['content-3'] = $replace['content-3'];

        $new['menu-life'] = $replace['menu-life'];
        $new['number-of-templates'] = $replace['number-of-templates'];
        $new['pdf-logo'] = $replace['pdf-logo'];

        $new['woocommerce-blocks'] = $replace['woocommerce-blocks'];
        $new['woocommerce-products-per-block'] = $replace['woocommerce-products-per-block'];

        $new['button-old-config'] = 1;

        update_option(self::OPTIONS, $new);

        if ($success):
            wp_send_json_success($new);
        endif;
    }

    public static function menu_life() {
        /*
          $titan = TitanFramework::getInstance(self::OPTIONS);
          $value = $titan->getOption(self::MENULIFE);
         */

        $options = get_option(self::OPTIONS);
        $value = $options[self::MENULIFE];
        return $value;
    }

    function default_expiry_date() {
        $day = new DateTime();
        $day->add(new DateInterval('P' . self::menu_life() . 'D'));
        $expiry = $day->format('U');
        return $expiry;
    }

    function do_meta_boxes() {
        $meta_boxes[] = array(
            "title" => __("Menu", "demomentsomtres-restaurant"),
            "desc" => __("Menu specific data", "demomentsomtres-restaurant"),
            "post_type" => array(self::POSTTYPE),
            "priority" => "high",
            'fields' => array(
                array(
                    "type" => "date",
                    "title" => __("Expiry date", "demomentsomtres-restaurant"),
                    "desc" => __("Dish list will be valid until the end of the selected day", "demomentsomtres-restaurant"),
                    "id" => self::EXPIRYDATE,
                    "default" => self::default_expiry_date(),
                ),
                array(
                    "type" => "date",
                    "title" => __("Hidden until", "demomentsomtres-restaurant"),
                    "desc" => __("Dish list will not be valid until the end of the selected day", "demomentsomtres-restaurant"),
                    "id" => self::DATAAMAGARFINS,
                    "default" => date("c"),
                ),
            )
        );
    }

    function posttypes() {
        $labels = array('name' => _x('Menu Types', 'taxonomy general name', "demomentsomtres-restaurant"), 'singular_name' => _x('Menu Type', 'taxonomy singular name', "demomentsomtres-restaurant"), 'search_items' => __('Search Type', "demomentsomtres-restaurant"), 'all_items' => __('All Types', "demomentsomtres-restaurant"), 'parent_item' => __('Parent Type', "demomentsomtres-restaurant"), 'parent_item_colon' => __('Parent Type:', "demomentsomtres-restaurant"), 'edit_item' => __('Edit Type', "demomentsomtres-restaurant"), 'update_item' => __('Update Type', "demomentsomtres-restaurant"), 'add_new_item' => __('Add New Type', "demomentsomtres-restaurant"), 'new_item_name' => __('New Type Name', "demomentsomtres-restaurant"),);
        register_taxonomy(self::TAXONOMY, '', array('hierarchical' => true, 'labels' => $labels));
        register_post_type(self::POSTTYPE, array('labels' => array('name' => _x('Menus', 'Post Type General Name', "demomentsomtres-restaurant"), 'singular_name' => _x('Menu', 'Post Type Singular Name', "demomentsomtres-restaurant"), 'menu_name' => __('Restaurant', "demomentsomtres-restaurant"), 'name_admin_bar' => __('Restaurant', "demomentsomtres-restaurant"), 'archives' => __('Restaurant menus', "demomentsomtres-restaurant"), 'parent_item_colon' => __('Parent menu:', "demomentsomtres-restaurant"), 'all_items' => __('All menus', "demomentsomtres-restaurant"), 'add_new_item' => __('Add new menu', "demomentsomtres-restaurant"), 'add_new' => __('Add New', "demomentsomtres-restaurant"), 'new_item' => __('New Menu', "demomentsomtres-restaurant"), 'edit_item' => __('Edit Menu', "demomentsomtres-restaurant"), 'update_item' => __('Update Menu', "demomentsomtres-restaurant"), 'view_item' => __('View Menu', "demomentsomtres-restaurant"), 'search_items' => __('Search Menu', "demomentsomtres-restaurant"), 'not_found' => __('Not found', "demomentsomtres-restaurant"), 'not_found_in_trash' => __('Not found in Trash', "demomentsomtres-restaurant"), 'featured_image' => __('Featured Image', "demomentsomtres-restaurant"), 'set_featured_image' => __('Set featured image', "demomentsomtres-restaurant"), 'remove_featured_image' => __('Remove featured image', "demomentsomtres-restaurant"), 'use_featured_image' => __('Use as featured image', "demomentsomtres-restaurant"), 'insert_into_item' => __('Insert into Menu', "demomentsomtres-restaurant"), 'uploaded_to_this_item' => __('Uploaded to this menu', "demomentsomtres-restaurant"), 'items_list' => __('Menus list', "demomentsomtres-restaurant"), 'items_list_navigation' => __('Menus list navigation', "demomentsomtres-restaurant"), 'filter_items_list' => __('Filter menus list', "demomentsomtres-restaurant"),), 'public' => true, 'show_in_nav_menus' => true, 'menu_position' => 15, 'taxonomies' => array('dish-list-type'), 'rewrite' => array('slug' => 'restaurant'), 'query_var' => true, 'has_archive' => true, 'supports' => array('title', 'editor', 'excerpt', 'trackbacks', 'custom-fields', 'comments', 'revisions', 'thumbnail', 'author', 'page-attributes')));
    }

    function admin_enqueue_scripts($hook) {
        wp_enqueue_script('dms3Restaurant', $this->pluginURL . 'js/demomentsomtres-restaurant.js');
        $translation_array = array();
        if (class_exists('Redux')):
            $redux = get_option(self::OPTIONS);
            $ntemp = $redux[self::TEMPLATES];
            $template_array = array();
            for ($i = 1; $i <= $ntemp; $i++) :
                $titol = $redux[self::TEMPLATETITLE . "-" . $i];
                $content = $redux[self::TEMPLATECONTENT . "-" . $i];
                $template_array[] = array('text' => $titol, 'value' => $content,);
            endfor;
            $translation_array = array(
                "templates" => $template_array,
                "title" => __("Add template", "demomentsomtres-restaurant"),
                'ajaxurl' => admin_url('admin-ajax.php')
            );
        endif;
        wp_localize_script('dms3Restaurant', 'dms3Restaurant', $translation_array);
    }

    /**
     * Checks if restaurant buttons are required
     * @since 1.0
     * @global string $current_screen
     */
    function insert_buttons() {
        global $current_screen;
        if ((!current_user_can('edit_posts') && !current_user_can('edit_pages')) && get_user_option('rich_editing')) :
            return;
        endif;
        if ($current_screen->post_type == "dish-list") :
            if (get_user_option('rich_editing')) :
                add_filter('mce_buttons', array($this, 'register_buttons'));
                add_filter("mce_external_plugins", array($this, "add_tinymce_plugin"));
            endif;
        endif;
    }

    /**
     * Registers buttons to TinyMCE
     * @param array $buttons
     * @return array
     * @since 1.0
     */
    function register_buttons($buttons) {
        array_push($buttons, "dms3RestaurantEco", "dms3RestaurantVeg", "dms3RestaurantCel", "dms3RestaurantPrice", "dms3RestaurantTemplate");
        return $buttons;
    }

    /**
     * Adds Load the TinyMCE plugin : editor_plugin.js
     * @param array $plugin_array
     * @return type
     * @since 1.0
     */
    function add_tinymce_plugin($plugin_array) {
        $plugin_array['dms3Restaurant'] = plugins_url('mce_plugins/editor_plugin.js', __FILE__);
        return $plugin_array;
    }

    function wp_footer() {
        /*
          if($this->printScripts):
          wp_enqueue_script("dms3okko");
          endif;
         */
    }

    /**
     * Manages dish list shortcode
     * @since 0.1
     * @param array $attr
     * @param string $content
     * @return string the Content with the dishlist
     */
    function shortcode_dish_list($attr) {
        if (!isset($attr['type'])) :
            return '';
        endif;
        $type = $attr['type'];
        $count = (!isset($attr['count'])) ? 1 : $attr['count'];
        $empty = (!isset($attr['empty'])) ? "" : $attr['empty'];
        $emptyurl = (!isset($attr['emptyurl'])) ? "" : $attr['emptyurl'];
        $prefix = (!isset($attr['prefix'])) ? "" : $attr['prefix'];
        $suffix = (!isset($attr['suffix'])) ? "" : $attr['suffix'];
        $classes = (!isset($attr['classes'])) ? "" : " " . $attr['classes'];
        $hiddenTitles = in_array('hidden_titles', $attr) ? true : false;
        //$hiddenTitles = (!isset($attr['hidden_titles'])) ? false : true;
        $blogRef = (!isset($attr['blog_id'])) ? FALSE : $attr['blog_id'];
        $titleFormat = (!isset($attr['title_format'])) ? 'h3' : $attr['title_format'];
        $cssid = (!isset($attr["id"])) ? "" : " id='" . $attr["id"] . "' ";
        $showPermalink = (in_array("permalink", $attr) || key_exists("permalink", $attr)) ? true : false;
        $permalinkText = (!isset($attr['permalink'])) ? __("link", "demomentsomtres-restaurant") : $attr['permalink'];
        $emptyjs = (!isset($attr['emptyjs'])) ? "" : $attr["emptyjs"];
        $hiddencontrol = (!isset($attr["hidden-control"])) ? false : $attr["hidden-control"];
        $wpml = (!isset($attr['wpml'])) ? "" : $attr['wpml'];
        if ($wpml):
            global $sitepress;
            $sitepress->switch_lang($wpml);
        endif;
        if (!$hiddencontrol):
            $allDishLists = $this->get_dish_lists($type, $count, $blogRef);
        else:
            $allDishLists = $this->get_dish_lists_with_hidden($type, $count, $blogRef);
        endif;
        $output = '';
        if (count($allDishLists) == 0) :
            if ($emptyurl == "") :
                $output .= $empty;
            else :
                $output .= "<a href='$emptyurl' title='$empty'>$empty</a>";
            endif;
        else :
            foreach ($allDishLists as $dishList) :
                $output .= "<div class='demomentsomtres-dish-list$classes'$cssid>";
                if (!$hiddenTitles) :
                    $output .= "<$titleFormat class=\"demomentsomtres-dish-list-title\">";
                    $output .= $prefix;
                    $output .= $dishList->post_title;
                    $output .= $suffix;
                    $output .= "</$titleFormat>";
                endif;
                $output .= wpautop($this->propagateID($dishList->post_content, $dishList->ID));
                if ($showPermalink):
//                    $output .= '<a href="' . get_permalink($dishList) . '" class="dms3-permalink" target="_blank">' . $permalinkText . "</a>";
                    $output .= "<a href='" . get_permalink($dishList) . "?action=" . self::PRINTMENU . "&lang=" . ICL_LANGUAGE_CODE . "' class='dms3-permalink' target='_blank'>" . $permalinkText . "</a>";
                endif;
                $output .= "</div>";
            endforeach;
        endif;
        if ($wpml):
            $sitepress->switch_lang(ICL_LANGUAGE_CODE);
        endif;
        return do_shortcode($output);
    }

    function shortcode_eco() {
        $output = '<i class="icon-eco">' . __('Ecological', "demomentsomtres-restaurant") . '</i>';
        return $output;
    }

    function shortcode_veg() {
        $output = '<i class="icon-veg">' . __('Vegetarian', "demomentsomtres-restaurant") . '</i>';
        return $output;
    }

    /**
     *
     * @since 1.4
     */
    function shortcode_cel() {
        $output = '<i class="icon-cel">' . __('Gluten Free', "demomentsomtres-restaurant") . '</i>';
        return $output;
    }

    function shortcode_p($attr) {
        $output = '<span class="price">' . $attr[0] . '</span>';
        //$output='<pre>'.print_r($attr,true).'</pre>';
        return $output;
    }

    /**
     * Modifies content post to insert the expiry date
     * @global mixed $post
     * @param text $content
     * @return string
     * @since 1.1.0
     */
    function content_expired_filter($content) {
        global $post;
        if (($post->post_type == 'dish-list') && in_the_loop()) :
            $newText = $this->expired_message($post->ID);
        else :
            $newText = '';
        endif;
        $content = $newText . $content;
        return $content;
    }

    /**
     * Generates a message to add to the dish list if it is expired
     * @param integer $postid the post id
     * @return string the message to attach
     * @since 1.1.0
     */
    function expired_message($postid) {
        if (self::is_expired($postid)) :
            $expiredMessage = __("Expired since", "demomentsomtres-restaurant") . " ";
            return '<span class="demomentsomtres-restaurant-expired">' . $expiredMessage . self::pretty_expiry_date($postid) . '</span>';
        else :
            return '';
        endif;
    }

    /**
     * Checks if the post is expired
     * @param integer $postid the post to test
     * @return boolean
     * @since 1.1.0
     */
    public static function is_expired($postid) {
        $expiry_date = self::get_expiry_date($postid);
        if ($expiry_date) :
            $now = strtotime(date('Y-m-d'));
            return ($now > $expiry_date);
        else :
            return false;
        endif;
    }

    /**
     * Get the expiry date based on postid
     * @param integer $postid the post to test
     * @return boolean
     * @since 1.1.0
     */
    public static function get_expiry_date($postid) {
        $redux = get_option(self::OPTIONS);
        $value = $redux[self::EXPIRYDATE];
        return $value;
    }

    public static function get_dataamagarfins($postid) {
        $redux = get_option(self::OPTIONS);
        $value = $redux[self::DATAAMAGARFINS];
        return $value;
    }

    /**
     * Formats the expiry date to be beautiful
     * @param integer $postid   the post to consider
     * @return string the date ready to be printed
     * @since 1.1.0
     */
    public static function pretty_expiry_date($postid) {
        $dateFormat = get_option('date_format');
        $dateDB = self::get_expiry_date($postid);
        $date = strtotime($dateDB);
        $text = date_i18n($dateFormat, $date);
        return $text;
    }

    public static function get_dish_lists($type, $count = 1, $blog = false) {
        $queryArgs = array(
            'post_type' => self::POSTTYPE,
            'orderby' => 'date',
            'order' => 'DESC',
            'posts_per_page' => $count,
            'tax_query' => array(
                array(
                    'taxonomy' => 'dish-list-type',
                    'field' => 'id',
                    'terms' => $type,
                ),
            ),
            'meta_query' => array(
                array(
                    'key' => self::OPTIONS . "_" . self::EXPIRYDATE,
                    'value' => strtotime(date('Y-m-d')),
                    'compare' => '>=',
                ),
            ),
        );
        if ($blog && is_multisite()) ://MQB 20151104+
            switch_to_blog($blog);
        endif;
        $newQuery = new WP_Query();
        $newQuery->query($queryArgs);
        if ($blog && is_multisite()) ://MQB 20151105+
            restore_current_blog();
        endif;
        return $newQuery->posts;
    }

    public static function get_dish_lists_with_hidden($type, $count = 1, $blog = false) {
        $queryArgs = array(
            'post_type' => self::POSTTYPE,
            'orderby' => 'date',
            'order' => 'DESC',
            'posts_per_page' => $count,
            'tax_query' => array(
                array(
                    'taxonomy' => 'dish-list-type',
                    'field' => 'id',
                    'terms' => $type,
                ),
            ),
            'meta_query' => array(
                array(
                    'key' => self::OPTIONS . "_" . self::EXPIRYDATE,
                    'value' => strtotime(current_time('Y-m-d')),
                    'compare' => '>=',
                ),
                array(
                    'key' => self::OPTIONS . "_" . self::DATAAMAGARFINS,
                    'value' => strtotime(current_time('Y-m-d H:i:s')),
                    'compare' => '<=',
                ),
            ),
        );
        if ($blog && is_multisite()) ://MQB 20151104+
            switch_to_blog($blog);
        endif;
        $newQuery = new WP_Query();
        $newQuery->query($queryArgs);
        if ($blog && is_multisite()) ://MQB 20151105+
            restore_current_blog();
        endif;
        return $newQuery->posts;
    }

    function shortcode_type($attr) {
        global $post;
        $attr = shortcode_atts(array(
            "id" => $post->ID,
            "wpml" => "",
                ), $attr);
        extract($attr);
        $output = "";
        if ($wpml):
            global $sitepress;
            $sitepress->switch_lang($wpml);
            $id = icl_object_id($post->ID, 'post', true, $wpml);
        endif;
        $terms = wp_get_post_terms($id, self::TAXONOMY);
        if (!isset($terms[0])):
            return;
        endif;
        $output .= "<div class='dishlist-type'>" . $terms[0]->name . "</div>";
        if ($wpml):
            $sitepress->switch_lang(ICL_LANGUAGE_CODE);
        endif;
        return $output;
    }

    function shortcode_post($attr) {
        global $post;
        $attr = shortcode_atts(array(
            "wpml" => "",
                ), $attr);
        extract($attr);
        $output = "<div class='demomentsomtres-restaurant-post'>";
        if ($wpml):
            global $sitepress;
            $sitepress->switch_lang($wpml);
            $id = icl_object_id($post->ID, 'post', true, $wpml);
        endif;
        $tpost = get_post($id);
        $output .= "<h2>" . $tpost->post_title . "</h2>";
        $output .= "<div class='entry-content'>";
        $output .= apply_filters("the_content", $tpost->post_content);
        $output .= "</div>";
        $output .= "</div>";
        if ($wpml):
            $sitepress->switch_lang(ICL_LANGUAGE_CODE);
        endif;
        return $output;
    }

    function template_redirect_printmenu($content) {
        $action = sanitize_text_field($_REQUEST[self::ACTION]);
        if ($action !== self::PRINTMENU):
            return;
        endif;

        $lang_base = ICL_LANGUAGE_CODE;
        $lang = $_REQUEST['lang'];
        do_action('wpml_switch_language', $lang);

        global $post;

        $langs = get_available_languages();
        $langs = array_merge($langs, get_available_languages($this->langDir));
        $langs = apply_filters('wpml_active_languages', NULL, 'orderby=id&order=desc');

        /* current language first */
        $current = $langs[$lang_base];
        unset($langs[$lang_base]);
        $langs = array_merge(array($lang_base => $current), $langs);

        $redux = get_option(self::OPTIONS);
        $css = $redux[self::TEMPLATEPRINTSTYLE];
        $imageID = $redux[self::PRINTLOGO];
        // The value may be a URL to the image (for the default parameter)
        // or an attachment ID to the selected image.
        $imageSrc = $imageID; // For the default value
        if (is_numeric($imageID)):
            $imageAttachment = wp_get_attachment_image_src($imageID, "medium");
            $imageSrc = $imageAttachment[0];
        endif;
        $logo = esc_url($imageSrc);
        $logo = str_replace("https://", "http://", $logo); //Evita el problema de les https que no mostren les imatges

        $html = "<html>";
        $html .= "<head>";
        $html .= "<style>$css</style>";
        $html .= "</head>";
        $html .= "<body>";

        foreach ($langs as $key => $l):
            $idtipusmenu = apply_filters('wpml_object_id', $post->ID, 'post', FALSE, $key);
            $tipus_menu = get_the_terms($idtipusmenu, self::TAXONOMY);
            $idpost = apply_filters('wpml_object_id', $post->ID, 'post', FALSE, $key);
            $tpost = get_post($idpost);

            $html .= "<div class='printable-content'>";
            $html .= '<div class="logo"><img src="' . $logo . '"/></div>';
            $html .= "<h2>" . $tipus_menu[0]->name . "</h2>";
            $html .= "<h3 class='entry-title'>" . $tpost->post_title . "</h3>";
            $html .= "<div class='entry-content'>";
            $html .= apply_filters('the_content', $tpost->post_content);
            $html .= "</div><!-- entry content -->";
            $html .= "</div><!-- printable content -->";
        endforeach;
        $html .= "</body>";
        $html .= "</html>";
        if (!$redux[self::PDFOUTPUT]):
            echo($html);
            die;
        else:
            $options = new Options();
            $options->set('isRemoteEnabled', true); //Troba imatges
            $options->set("dpi", 300);
            $dompdf = new Dompdf($options);

            $dompdf->setPaper('A4');
            $dompdf->loadHtml($html);
            $dompdf->render(); //Renderitzem HTML a PDF

            $dompdf->stream($this->fitxer($tipus_menu[0]->name), array("Attachment" => false)); //Exportar PDF com a fitxer en pantalla
//          $dompdf->stream($this->fitxer($tipus_menu[0]->name), array("Attachment" => true)); //Exportar PDF com a fitxer descarregable
            die;
        endif;
    }

    function fitxer($titol) {
        return str_replace(" ", "-", strtolower($titol)) . ".pdf";
    }

    function propagateID($content, $post_id) {
        $content = str_replace("[" . self::SHORTCODE_WC, "[" . self::SHORTCODE_WC . " postid='$post_id'", $content);
        return $content;
    }

    function shortcode_woocommerce($atts) {
        global $post;
        $atts = shortcode_atts(
                array(
                    "block" => 0,
                    "class" => "",
                    "postid" => $post->ID,
                    "columns" => 1,
                ),
                $atts
        );
        $columns = $atts["columns"];
        $output = "";
        if (class_exists('WooCommerce')):
            $redux = get_option(self::OPTIONS);
            $nwcb = $redux[self::OPTION_WC_BLOCKS];
            $nwcpb = $redux[self::OPTION_WC_PRODUCTS_BLOCK];
            $products = [];
            $i = $atts["block"] - 1;
            for ($j = 0; $j < $nwcpb; $j++):
                $prodid = get_post_meta($atts["postid"], self::OPTIONS . "_" . self::FIELD_WC_PRODUCTS . "-$i-$j", true);
                if ($prodid):
                    $products[] = $prodid;
                endif;
            endfor;
            $products = implode(",", $products);
            $output .= "<div class='" . $atts["class"] . "'>";
            if ($products):
                $output .= "[products columns='$columns' limit='$nwcpb' ids='$products']";
            endif;
            $output .= "</div>";
        endif;
        return do_shortcode($output);
    }

}
